// FindSheet.cpp : implementation file
//

#include "stdafx.h"
#include "ModalDemo.h"
#include "FindSheet.h"
#include ".\findsheet.h"

#define RESULT_TAB_INDEX 1
#define RESULT_TAB_CAPTION "Result: %ld books found"

// CFindSheet

IMPLEMENT_DYNAMIC(CFindSheet, CPropertySheet)
CFindSheet::CFindSheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(nIDCaption, pParentWnd, iSelectPage)
{
}

CFindSheet::CFindSheet(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
}

CFindSheet::~CFindSheet()
{
}


BEGIN_MESSAGE_MAP(CFindSheet, CPropertySheet)
END_MESSAGE_MAP()


// CFindSheet message handlers

BOOL CFindSheet::OnInitDialog()
{
	BOOL bResult = CPropertySheet::OnInitDialog();

	// TODO:  Add your specialized code here

	int ids[] = {	IDCANCEL,
					ID_APPLY_NOW };
	int len = sizeof(ids) / sizeof(ids[0]);
	for(int i = 0; i < len; i++){
		CWnd *pWnd = GetDlgItem(ids[i]);
		ASSERT(pWnd);
		if(pWnd) 
			pWnd->ShowWindow(FALSE);
	}

	CWnd *pBtnOk = GetDlgItem(IDOK);
	ASSERT(pBtnOk);

	CRect rectSheet;
	GetWindowRect(rectSheet); //   

	CRect rectBtnOk;
	pBtnOk->GetWindowRect(rectBtnOk); //   

	//      
	//     
	int iBorder = rectSheet.bottom - rectBtnOk.bottom;

	//    
	rectSheet.right += rectBtnOk.Width() + iBorder;
	rectSheet.bottom = rectBtnOk.top;
	MoveWindow(rectSheet);

	//   
	CPropertyPage *pPage = GetPage(0);
	ASSERT(pPage);
	CRect rectPage;
	pPage->GetWindowRect(rectPage); //   

	//      OK
	int cxOk = rectBtnOk.Width();
	int cyOk = rectBtnOk.Height();
	
	//   OK
	rectBtnOk.top = rectPage.top;
	rectBtnOk.bottom = rectPage.top + cyOk;
	rectBtnOk.left = rectSheet.right - (cxOk + iBorder);
	rectBtnOk.right = rectBtnOk.left + cxOk;
	ScreenToClient(rectBtnOk);
	pBtnOk->MoveWindow(rectBtnOk);

	pBtnOk->SetWindowText(_T("Close"));

	DisablePage(1, -1);

	return bResult;
}

//  
void CFindSheet::DisablePage(int iFirstPage, ...)
{
	int iPage = iFirstPage;

	va_list marker;
	va_start (marker, iFirstPage);
	int nArgs = 0;
	while(iPage != -1){//   -1
		//    
		//    , 
		//    
		int count = (int)m_arrDisabledPages.GetCount();
		bool flag = false;
		for(int i = 0; i < count; i++)
			if(m_arrDisabledPages.GetAt(i) == (UINT)iPage){
				flag = true;
				break;
			}
		if(!flag)
			m_arrDisabledPages.Add(iPage);

		SetDisabledText(iPage);	//    

		//    
		iPage = va_arg(marker, UINT);
		nArgs++; //  
		ASSERT(nArgs < 100); //  
	}
}

#define DISABLED_TEXT " - Disabled"
//    
void CFindSheet::SetDisabledText(int iPage)
{
	CTabCtrl *pTab = GetTabControl();
	ASSERT(pTab);
	TC_ITEM ti;
	char szText[100];
	ti.mask = TCIF_TEXT;
	ti.pszText = szText;
	ti.cchTextMax = 100;
	VERIFY(pTab->GetItem(iPage, &ti));
	strcat(szText, DISABLED_TEXT);
	VERIFY(pTab->SetItem(iPage, &ti));
}

BOOL CFindSheet::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	NMHDR *pNmh = (NMHDR*)lParam;
	ASSERT(pNmh);
	if(pNmh->code == TCN_SELCHANGING)
		m_iLastActivePage = GetActiveIndex();
	else if(pNmh->code == TCN_SELCHANGE){
		int iCurrPage = GetActiveIndex();
		if(IsPageDisabled(iCurrPage))
			PostMessage(PSM_SETCURSEL, m_iLastActivePage);
	}

	return CPropertySheet::OnNotify(wParam, lParam, pResult);
}

// ,       
BOOL CFindSheet::IsPageDisabled(int iPage)
{
	BOOL bFoundEntry = FALSE;
	int iSize = (int)m_arrDisabledPages.GetSize();

	int i = 0;
	while(i < iSize && !bFoundEntry)
		if(m_arrDisabledPages.GetAt(i) == (UINT)iPage)
			bFoundEntry = TRUE;
		else
			i++;

	return bFoundEntry;
}

//  
void CFindSheet::EnablePage(int iPage)
{
	BOOL bFoundEntry = FALSE;
	int iSize = (int)m_arrDisabledPages.GetSize();

	int i = 0;
	while(i < iSize && !bFoundEntry)
		if(m_arrDisabledPages.GetAt(i) == (UINT)iPage)
			bFoundEntry = TRUE;
		else
			i++;

	if(bFoundEntry){
		m_arrDisabledPages.RemoveAt(i);
		SetEnabledText(iPage);
	}
}


//   " - Disabled"   
void CFindSheet::SetEnabledText(int iPage)
{
	CTabCtrl *pTab = GetTabControl();
	ASSERT(pTab);
	TC_ITEM ti;
	char szText[100];
	ti.mask = TCIF_TEXT;
	ti.pszText = szText;
	ti.cchTextMax = 100;
	VERIFY(pTab->GetItem(iPage, &ti));
	//  strstr()     szText
	//    DISABLED_TEXT
	char *pFound = strstr(szText, DISABLED_TEXT);
	if(pFound){
		*pFound = '\0'; //    
		VERIFY(pTab->SetItem(iPage, &ti));
	}
}

//      
void CFindSheet::SetResult(int nHits)
{
	//    
	CTabCtrl *pTab = GetTabControl();
	ASSERT(pTab);
	TC_ITEM ti;
	char szText[100];
	ti.mask = TCIF_TEXT;
	ti.pszText = szText;
	ti.cchTextMax = 100;
	VERIFY(pTab->GetItem(RESULT_TAB_INDEX, &ti));
	sprintf(szText, RESULT_TAB_CAPTION, nHits);
	VERIFY(pTab->SetItem(RESULT_TAB_INDEX, &ti));

	//   
	if(nHits <= 0)
		DisablePage(RESULT_TAB_INDEX, -1);
	else
		EnablePage(RESULT_TAB_INDEX);
}
